import gleam/dynamic/decode.{type Decoder}
import models/faction.{type Faction}
import models/faction_symbol.{type FactionSymbol}
import utils/api.{type ApiResponse, type PagedData}
import utils/auth.{type AgentToken, AgentAuth}

pub type ListFactionsResponse {
  ListFactionsResponse(factions: List(Faction))
}

fn list_factions_response_decoder() -> Decoder(ListFactionsResponse) {
  use factions <- decode.then(decode.list(faction.decoder()))
  decode.success(ListFactionsResponse(factions:))
}

pub fn list_factions(
  token: AgentToken,
  page: Int,
  limit: Int,
) -> ApiResponse(PagedData(ListFactionsResponse)) {
  let request = api.get_page(AgentAuth(token), "/factions", page, limit)
  use response <- api.try_send(request)
  case response.status {
    200 ->
      api.parse_paged_data_response(response, list_factions_response_decoder())
    _ -> api.parse_error_response(response)
  }
}

pub type GetFactionResponse {
  GetFactionResponse(faction: Faction)
}

fn get_faction_response_decoder() -> Decoder(GetFactionResponse) {
  use faction <- decode.then(faction.decoder())
  decode.success(GetFactionResponse(faction:))
}

pub fn get_faction(
  token: AgentToken,
  symbol: FactionSymbol,
) -> ApiResponse(GetFactionResponse) {
  let request =
    api.get(AgentAuth(token), "/factions/" <> faction_symbol.to_string(symbol))
  use response <- api.try_send(request)
  case response.status {
    200 -> api.parse_data_response(response, get_faction_response_decoder())
    _ -> api.parse_error_response(response)
  }
}

pub type FactionReputation {
  FactionReputation(symbol: FactionSymbol, reputation: Int)
}

fn faction_reputation_decoder() {
  use symbol <- decode.field("symbol", faction_symbol.decoder())
  use reputation <- decode.field("reputation", decode.int)
  decode.success(FactionReputation(symbol:, reputation:))
}

pub type GetMyFactionsResponse {
  GetMyFactionsResponse(faction_reputations: List(FactionReputation))
}

fn get_my_factions_response_decoder() -> Decoder(GetMyFactionsResponse) {
  use faction_reputations <- decode.then(
    decode.list(faction_reputation_decoder()),
  )
  decode.success(GetMyFactionsResponse(faction_reputations:))
}

pub fn get_my_factions(
  token: AgentToken,
  page: Int,
  limit: Int,
) -> ApiResponse(PagedData(GetMyFactionsResponse)) {
  let request = api.get_page(AgentAuth(token), "/my/factions", page, limit)
  use response <- api.try_send(request)
  case response.status {
    200 ->
      api.parse_paged_data_response(
        response,
        get_my_factions_response_decoder(),
      )
    _ -> api.parse_error_response(response)
  }
}
