import dot_env
import dot_env/env
import gleam/result
import utils/auth.{type AccountToken, type AgentToken}

pub type Env {
  Env(account_token: AccountToken, agent_token: AgentToken)
}

pub type EnvError {
  MissingVar(name: String)
  InvalidVar(name: String)
}

fn get_string(name: String) -> Result(String, EnvError) {
  env.get_string(name) |> result.replace_error(MissingVar(name))
}

pub fn load() -> Result(Env, EnvError) {
  dot_env.default |> dot_env.load
  use account_token <- result.try(
    get_string("ACCOUNT_TOKEN")
    |> result.then(fn(token_str) {
      auth.parse_account_token(token_str)
      |> result.replace_error(InvalidVar("ACCOUNT_TOKEN"))
    }),
  )
  use agent_token <- result.try(
    get_string("AGENT_TOKEN")
    |> result.then(fn(token_str) {
      auth.parse_agent_token(token_str)
      |> result.replace_error(InvalidVar("AGENT_TOKEN"))
    }),
  )
  Ok(Env(account_token:, agent_token:))
}

pub fn load_unsafe() -> Env {
  let assert Ok(env) = load() as "Failed to load environment variables"
  env
}
